import { ComposeSpecification, ListOrDict } from "../compose-schema.ts";
import { registerDeployment } from "../deployment.ts";
import { relative } from "../path.ts";
import { Network } from "./network.ts";
import { Service } from "./service.ts";
import { Volume } from "./volume.ts";

function mapRecordToCompose<T, F extends { toCompose(): T }>(
  obj: Record<string, F>
): Record<string, T> {
  return Object.fromEntries(
    Object.entries(obj).map(([k, v]) => [k, v.toCompose()])
  );
}

/**
 * ComposeStack is a collection of services, networks, and volumes.
 * Every stack gets turned into a compose file at the end of the execution.
 * To create a stack, define a new class that extends Stack, then
 * instantiate the class with the name of the stack.
 *
 * Example:
 * ```ts
 * import { Stack } from "../mod.ts";
 *
 * class ExampleStack extends Stack {
 *   constructor(name: string) {
 *     super(import.meta.url, name);
 *
 *     const redis = this.addService("redis", {
 *       image: "registry.hub.docker.com/library/redis:6.2-alpine",
 *       restart: "unless-stopped",
 *     });
 *   }
 * }
 *
 * const stack = new ExampleStack("example");
 * ```
 */
export class ComposeStack {
  private services: Record<string, Service> = {};
  private networks: Record<string, Network> = {};
  private volumes: Record<string, Volume> = {};

  constructor(public readonly stackPath: string, public name: string) {
    registerDeployment(this);
  }

  public addService(service: Service): Service {
    const name = service.name;

    if (this.services[name]) {
      throw new Error(`Service ${name} already defined.`);
    }

    // Inject stax labels
    this.injectLabels(service);

    this.services[name] = service;
    return this.services[name];
  }

  public addNetwork(network: Network): Network {
    const name = network.name;

    if (this.networks[name]) {
      throw new Error(`Network ${name} already defined.`);
    }

    // Inject stax labels if not external
    if (!network.options.external) {
      this.injectLabels(network);
    }

    this.networks[name] = network;
    return this.networks[name];
  }

  public addVolume(volume: Volume): Volume {
    const name = volume.name;

    if (this.volumes[name]) {
      throw new Error(`Volume ${name} already defined.`);
    }

    // Inject stax labels
    this.injectLabels(volume);

    this.volumes[name] = volume;
    return this.volumes[name];
  }

  public toComposeFile(): ComposeSpecification {
    return {
      version: "3.8",
      services: mapRecordToCompose(this.services),
      networks: mapRecordToCompose(this.networks),
      volumes: mapRecordToCompose(this.volumes),
    };
  }

  private injectLabels<
    T extends { options: { labels?: ListOrDict | undefined } }
  >(resourse: T) {
    resourse.options.labels = {
      ...resourse.options.labels,
      ...{
        "com.stax.stack": this.name,
        "com.stax.stack-path": relative(this.stackPath),
      },
    };
  }
}
